@@ -59,7 +59,11 @@ from aiohomekit.utils import async_create_task
 from aiohomekit.uuid import normalize_uuid
 
 from ..abstract import AbstractPairing, AbstractPairingData
-from .bleak import AIOHomeKitBleakClient
+from .bleak import (
+    AIOHomeKitBleakClient,
+    BleakCharacteristicMissing,
+    BleakServiceMissing,
+)
 from .client import (
     PDUStatusError,
     ble_request,
@@ -157,6 +161,31 @@ def operation_lock(func: WrapFuncType) -> WrapFuncType:
     return cast(WrapFuncType, _async_operation_lock_wrap)
 
 
+def disconnect_on_missing_services(func: WrapFuncType) -> WrapFuncType:
+    """Define a wrapper to disconnect on missing services and characteristics.
+
+    This must be placed after the retry_bluetooth_connection_error
+    decorator.
+    """
+
+    async def _async_disconnect_on_missing_services_wrap(
+        self: BlePairing, *args: Any, **kwargs: Any
+    ) -> None:
+        try:
+            return await func(self, *args, **kwargs)
+        except (BleakServiceMissing, BleakCharacteristicMissing) as ex:
+            logger.warning(
+                "%s: Missing service or characteristic, disconnecting to force refetch of GATT services: %s",
+                self.name,
+                ex,
+            )
+            if self.client:
+                await self.client.disconnect()
+            raise
+
+    return cast(WrapFuncType, _async_disconnect_on_missing_services_wrap)
+
+
 def restore_connection_and_resume(func: WrapFuncType) -> WrapFuncType:
     """Define a wrapper restore connection, populate data, and then resume when the operation completes."""
 
@@ -527,6 +556,7 @@ class BlePairing(AbstractPairing):
 
     @operation_lock
     @retry_bluetooth_connection_error()
+    @disconnect_on_missing_services
     @restore_connection_and_resume
     async def _process_disconnected_events_with_retry(
         self,
@@ -618,8 +648,11 @@ class BlePairing(AbstractPairing):
             return
 
         logger.warning(
-            "%s: Received notification but could not decrypt: %s", self.name, data
+            "%s: Received notification but could not decrypt, falling back processing as disconnected event: %s",
+            self.name,
+            data,
         )
+        async_create_task(self._process_disconnected_events())
 
     def _async_get_service_signature_char(self) -> Characteristic | None:
         """Get the service signature characteristic."""
@@ -827,10 +860,35 @@ class BlePairing(AbstractPairing):
 
     @operation_lock
     @retry_bluetooth_connection_error()
+    @disconnect_on_missing_services
     @restore_connection_and_resume
     async def list_accessories_and_characteristics(self) -> list[dict[str, Any]]:
         return self.accessories.serialize()
 
+    async def get_primary_name(self) -> str:
+        """Return the primary name of the device.
+
+        This overrides the default implementation
+        to get name from the advertisement if the
+        accessories are not available which happens
+        frequently with the ESPHome proxies because
+        they cannot currently connect right away again
+        after disconnecting.
+
+        We want to avoid raising here if possible
+        since this is called right after pairing
+        and if we raise the pairing will fail but
+        the device will still be paired and the user
+        will have to un-pair it manually with a factory
+        reset.
+        """
+        if not self.accessories and self.description:
+            self._accessories_state = AccessoriesState(
+                Accessories(), -1, self.broadcast_key
+            )
+            return self.description.name
+        return await super().get_primary_name()
+
     async def _populate_char_values(self, config_changed: bool) -> None:
         """Populate the values of all characteristics."""
         chars: list[Characteristic] = []
@@ -840,6 +898,7 @@ class BlePairing(AbstractPairing):
             if (
                 not config_changed
                 and service.type == ServicesTypes.ACCESSORY_INFORMATION
+                and service.value(CharacteristicsTypes.NAME)
             ):
                 continue
             for char in service.characteristics:
@@ -927,6 +986,7 @@ class BlePairing(AbstractPairing):
 
     @operation_lock
     @retry_bluetooth_connection_error()
+    @disconnect_on_missing_services
     async def _async_populate_accessories_state(
         self, force_update: bool = False, attempts: int | None = None
     ) -> None:
@@ -941,10 +1001,13 @@ class BlePairing(AbstractPairing):
         Older code did not save the handle, so if we have no handles
         we need to re-fetch the gatt database.
         """
-        for service in self.accessories.aid(BLE_AID).services:
-            for char in service.characteristics:
-                if char.handle is not None:
-                    return False
+        try:
+            for service in self.accessories.aid(BLE_AID).services:
+                for char in service.characteristics:
+                    if char.handle is not None:
+                        return False
+        except StopIteration:
+            return True
         return True
 
     async def _populate_accessories_and_characteristics(
@@ -1112,6 +1175,7 @@ class BlePairing(AbstractPairing):
 
     @operation_lock
     @retry_bluetooth_connection_error()
+    @disconnect_on_missing_services
     async def _process_config_changed(self, config_num: int) -> None:
         """Process a config change.
 
@@ -1123,6 +1187,7 @@ class BlePairing(AbstractPairing):
 
     @operation_lock
     @retry_bluetooth_connection_error()
+    @disconnect_on_missing_services
     @restore_connection_and_resume
     async def list_pairings(self):
         request_tlv = TLV.encode_list(
@@ -1164,6 +1229,7 @@ class BlePairing(AbstractPairing):
         return tmp
 
     @retry_bluetooth_connection_error()
+    @disconnect_on_missing_services
     async def get_characteristics(
         self,
         characteristics: list[tuple[int, int]],
@@ -1272,6 +1338,7 @@ class BlePairing(AbstractPairing):
 
     @operation_lock
     @retry_bluetooth_connection_error()
+    @disconnect_on_missing_services
     @restore_connection_and_resume
     async def put_characteristics(
         self, characteristics: list[tuple[int, int, Any]]
@@ -1348,6 +1415,7 @@ class BlePairing(AbstractPairing):
 
     @operation_lock
     @retry_bluetooth_connection_error()
+    @disconnect_on_missing_services
     @restore_connection_and_resume
     async def _async_subscribe(self, new_chars: Iterable[tuple[int, int]]) -> None:
         """Subscribe to new characteristics."""
@@ -1390,6 +1458,7 @@ class BlePairing(AbstractPairing):
 
     @operation_lock
     @retry_bluetooth_connection_error()
+    @disconnect_on_missing_services
     @restore_connection_and_resume
     async def add_pairing(
         self, additional_controller_pairing_identifier, ios_device_ltpk, permissions
@@ -1446,6 +1515,7 @@ class BlePairing(AbstractPairing):
 
     @operation_lock
     @retry_bluetooth_connection_error(attempts=10)
+    @disconnect_on_missing_services
     @restore_connection_and_resume
     async def remove_pairing(self, pairingId: str) -> bool:
         request_tlv = TLV.encode_list(
